/* converter.cc
 * This file is part of mt-utils library
 * Copyright (c) 2004 by Stanislav Ievlev
 *
 * This file is covered by the GNU Library General Public License,
 * which should be included with mt-utils as the file COPYING.
 */

#include <stdexcept>
#include <vector>

#include "converter.hh"

converter::converter(mt::lang_code langnum)
{
     const std::string& int_cset = std::use_facet<mt::identification>(*mt::get_locale(langnum)).codeset();
     const std::string& ext_cset = std::use_facet<mt::identification>(*mt::get_locale("")).codeset();

    int_ext_ = iconv_open( ext_cset.c_str(), int_cset.c_str() );
    if ( reinterpret_cast<iconv_t>(-1) == int_ext_ )
	    throw std::runtime_error("unable to open iconv from "+int_cset+" to" + ext_cset);
    ext_int_ = iconv_open( int_cset.c_str(), ext_cset.c_str() );
    if ( reinterpret_cast<iconv_t>(-1) == ext_int_ )
        throw std::runtime_error("unable to open iconv from "+ext_cset+" to" + int_cset);
}
    
converter::~converter()
{
    iconv_close( int_ext_ );
    iconv_close( ext_int_ );
}
    
std::string converter::in(const std::string& str) const
{
    return tr(str,ext_int_);
}

std::string converter::out(const std::string& str) const
{
    return tr(str,int_ext_);
}

std::string converter::tr(const std::string& message, iconv_t id) const
{


    std::vector<char> in(message.begin(),message.end());
    in.push_back(0);
    std::vector<char> out(message.length()*8);

    std::fill(out.begin(), out.end(), 0);

    size_t in_len = in.size();
    size_t out_len = out.size();
    char *in_ptr = &in[0];
    char *out_ptr = &out[0];


    if ( iconv( id, &in_ptr, &in_len, &out_ptr, &out_len ) == static_cast<size_t>(-1) )
	    return message;


    return std::string(out.begin(),out.end());
}
